@@ -1,17 +1,21 @@
 import json
 import requests
 import logging
+import sched
+import threading
+import time
 from dateutil.relativedelta import *
 
 from datetime import datetime
 from dateutil import parser
-
+from threading import Event
 
 class Device(object):
     """ Base class definition of a device in the alarm system. """
 
     # Property variables
     __id = None
+    __name = None
     __zone = None
     __device_type = None
     __subtype = None
@@ -19,11 +23,12 @@ class Device(object):
     __warnings = None
     __partitions = None
 
-    def __init__(self, id, zone, device_type, subtype,
+    def __init__(self, id, name, zone, device_type, subtype,
                  preenroll, warnings, partitions):
         """ Set the private variable values on instantiation. """
 
         self.__id = id
+        self.__name = name
         self.__zone = zone
         self.__device_type = device_type
         self.__subtype = subtype
@@ -37,6 +42,11 @@ class Device(object):
         """ Device ID. """
         return self.__id
 
+    @property
+    def name(self):
+        """ Device Name. """
+        return self.__name
+
     @property
     def zone(self):
         """ Device zone. """
@@ -74,10 +84,11 @@ class ContactDevice(Device):
     @property
     def state(self):
         """ Returns the current state of the contact. """
-
         if self.warnings:
-            if 'OPENED' in self.warnings:
+            if 'OPENED' in str(self.warnings):
                 return 'opened'
+            else:
+                return 'closed'
         else:
             return 'closed'
 
@@ -120,20 +131,36 @@ class System(object):
     __api = None
 
     # Property variables
-    __system_name = None
     __system_serial = None
     __system_model = None
     __system_ready = False
     __system_state = None
     __system_alerts = None
     __system_troubles = None
-    __system_active = False
     __system_connected = False
+    __system_alarm = False
     __system_devices = []
+    __scheduler = None
+    __scheduler_thread = None
+    __scheduler_event = None
 
-    def __init__(self, hostname, user_code, user_id, user_email, user_password, panel_id, partition):
+    def __init__(self, hostname, app_id, user_code, user_email, user_password, panel_id, partition):
         """ Initiate the connection to the Visonic API """
-        self.__api = API(hostname, user_code, user_id, user_email, user_password, panel_id, partition)
+        self.__api = API(hostname, app_id, user_code, user_email, user_password, panel_id, partition)
+
+        #Create schedule
+        self.__scheduler = sched.scheduler(time.time, time.sleep)
+        self.__scheduler_thread = threading.Thread(target=self.__scheduler.run, daemon=True, name="state_schedule_thread")
+
+    def _run_scheduled_func(self, timer_seconds, timer_function, token, func):
+        timer_function(token, func)
+        self.schedule(timer_seconds, timer_function, token, func)
+
+    def schedule(self, timer_seconds, timer_function, token, func):
+        self.__scheduler.enter(timer_seconds, 1, self._run_scheduled_func,(timer_seconds, timer_function, token, func))
+
+        if not self.__scheduler_thread.isAlive():
+            self.__scheduler_thread.start()
 
     # System properties
     @property
@@ -141,11 +168,6 @@ class System(object):
         """ Serial number of the system. """
         return self.__system_serial
 
-    @property
-    def name(self):
-        """ Name of the system. """
-        return self.__system_name
-
     @property
     def model(self):
         """ Model of the system. """
@@ -167,11 +189,6 @@ class System(object):
         """ Current alarm of the alarm system. """
         return self.__system_alarm
 
-    @property
-    def active(self):
-        """ If the alarm system is active or not. """
-        return self.__system_active
-
     @property
     def is_token_valid(self):
         """ If the alarm system is active or not. """
@@ -201,15 +218,44 @@ class System(object):
 
     def disarm(self):
         """ Send Disarm command to the alarm system. """
-        self.__api.disarm(self.__api.partition)
+        res = self.__api.disarm(self.__api.partition)
+
+        self.ProcessStatus(res['process_token'], "DISARM")
 
     def arm_home(self):
         """ Send Arm Home command to the alarm system. """
-        self.__api.arm_home(self.__api.partition)
+        res = self.__api.arm_home(self.__api.partition)
+        
+        self.ProcessStatus(res['process_token'], "HOME")
 
     def arm_away(self):
         """ Send Arm Away command to the alarm system. """
-        self.__api.arm_away(self.__api.partition)
+        res = self.__api.arm_away(self.__api.partition)
+        
+        self.ProcessStatus(res['process_token'], "AWAY")
+
+    def ProcessStatus(self, token, function):
+        logging.debug("Calling get_process_status with token " + token)
+        res = self.__api.get_process_status(token)
+        
+        if res == 'handled':
+            if function == 'AWAY':
+                self.__system_state = 'EXITDELAYAWAY'
+            elif function == 'HOME':
+                self.__system_state = 'EXITDELAYHOME'
+            elif function == 'DISARM':
+                self.__system_state = 'ENTRYDELAY'
+
+            time.sleep(1)
+            self.ProcessStatus(token, function)
+        elif res == 'succeeded':
+            self.update_status()
+            
+            logging.debug("Succeded.. updating status" )
+            return
+        else:
+            logging.debug("Stuck processstatus?")
+            return
 
     def connect(self):
         """ Connect to the alarm system and get the static system info. """
@@ -222,16 +268,17 @@ class System(object):
         else:
             raise Exception('Rest API version 8.0 is not supported by server.')
 
+        # Try to login and get a user token.
         self.__api.login()
-        print('Login successful.')
+        logging.debug('Login successful')
 
-        # Try to login and get a session token.
+        # Try to panel login and get a session token.
         # This will raise an exception on failure.
         self.__api.panel_login()
-        print('Panel Login successful.')
+        logging.debug('Panel Login successful')
 
         # Get general panel information
-        gpi = self.__api.get_general_panel_info()
+        gpi = self.__api.get_panel_info()
         self.__system_serial = gpi['serial']
         self.__system_model = gpi['model']
 
@@ -284,7 +331,7 @@ class System(object):
         print('---------------------------------')
         print('Host:          {0}'.format(self.__api.hostname))
         print('User Code:     {0}'.format(self.__api.user_code))
-        print('User ID:       {0}'.format(self.__api.user_id))
+        print('App ID:       {0}'.format(self.__api.app_id))
         print('Panel ID:      {0}'.format(self.__api.panel_id))
         print('Partition:     {0}'.format(self.__api.partition))
         print('Session-Token: {0}'.format(self.__api.session_token))
@@ -293,12 +340,10 @@ class System(object):
         print('----------------------------')
         print(' General system information ')
         print('----------------------------')
-        print('Name:         {0}'.format(self.__system_name))
         print('Serial:       {0}'.format(self.__system_serial))
         print('Model:        {0}'.format(self.__system_model))
         print('Ready:        {0}'.format(self.__system_ready))
         print('State:        {0}'.format(self.__system_state))
-        print('Active:       {0}'.format(self.__system_active))
         print('Connected:    {0}'.format(self.__system_connected))
 
     def print_system_devices(self, detailed=False):
@@ -310,10 +355,11 @@ class System(object):
             print(' Device #{0} '.format(index+1))
             print('--------------')
             print('ID:             {0}'.format(device.id))
+            print('Name:           {0}'.format(device.name))
             print('Zone:           {0}'.format(device.zone))
             print('Device Type:    {0}'.format(device.device_type))
             print('Subtype:        {0}'.format(device.subtype))
-            print('Warnings:         {0}'.format(device.warnings))
+            print('Warnings:       {0}'.format(device.warnings))
 
             if detailed:
                 print('Pre-enroll:     {0}'.format(device.pre_enroll))
@@ -345,7 +391,7 @@ class System(object):
     def update_status(self):
         """ Update all variables that are populated by the call
         to the status() API method. """
-
+    
         status = self.__api.get_status()
         partition = status['partitions'][0]
 
@@ -360,8 +406,8 @@ class System(object):
         else:
             state = partition['state']
             self.__system_alarm = True
-            if state == 'Home' or state == 'Home Instant' or state == 'Away' or state == 'Away Instant':
-                self.__system_state = 'Alarm'
+            if state == 'HOME' or state == 'AWAY':
+                self.__system_state = 'ALARM'
             else:
                 self.__system_state = partition['state']
 
@@ -384,6 +430,7 @@ class System(object):
             if device['subtype'] == 'CONTACT_AUX' or device['subtype'] == 'CONTACT':
                 contact_device = ContactDevice(
                     id=device['id'],
+                    name=device['name'],
                     zone=device['zone_type'],
                     device_type=device['device_type'],
                     subtype=device['subtype'],
@@ -395,6 +442,7 @@ class System(object):
             elif device['subtype'] == 'MOTION_CAMERA':
                 camera_device = CameraDevice(
                     id=device['id'],
+                    name=device['name'],
                     zone=device['zone_type'],
                     device_type=device['device_type'],
                     subtype=device['subtype'],
@@ -406,6 +454,7 @@ class System(object):
             elif device['subtype'] == 'MOTION' or device['subtype'] == 'CURTAIN':
                 motion_device = MotionDevice(
                     id=device['id'],
+                    name=device['name'],
                     zone=device['zone_type'],
                     device_type=device['device_type'],
                     subtype=device['subtype'],
@@ -417,6 +466,7 @@ class System(object):
             elif device['subtype'] == 'SMOKE':
                 smoke_device = SmokeDevice(
                     id=device['id'],
+                    name=device['name'],
                     zone=device['zone_type'],
                     device_type=device['device_type'],
                     subtype=device['subtype'],
@@ -428,6 +478,7 @@ class System(object):
             else:
                 generic_device = GenericDevice(
                     id=device['id'],
+                    name=device['name'],
                     zone=device['zone_type'],
                     device_type=device['device_type'],
                     subtype=device['subtype'],
@@ -447,9 +498,9 @@ class API(object):
     __rest_version = '8.0'
     __hostname = 'visonic.tycomonitor.com'
     __user_code = '1234'
-    __user_id = '00000000-0000-0000-0000-000000000000'
+    __app_id = '00000000-0000-0000-0000-000000000000'
     __panel_id = '123456'
-    __partition = 'ALL'
+    __partition = '-1'
     __user_email = 'example@example.com'
     __user_password = 'example'
 
@@ -462,12 +513,13 @@ class API(object):
     __url_alarms = None
     __url_alerts = None
     __url_troubles = None
-    __url_general_panel_info = None
+    __url_panel_info = None
     __url_events = None
     __url_wakeup_sms = None
     __url_all_devices = None
     __url_set_state = None
     __url_locations = None
+    __url_process_status = None
 
     # API session token
     __session_token = None
@@ -476,14 +528,15 @@ class API(object):
     # Use a session to reuse one TCP connection instead of creating a new
     # connection for every call to the API
     __session = None
+    __system_devices = []
 
-    def __init__(self, hostname, user_code, user_id, user_email, user_password, panel_id, partition):
+    def __init__(self, hostname, app_id, user_code, user_email, user_password, panel_id, partition):
         """ Class constructor initializes all URL variables. """
 
         # Set connection specific details
         self.__hostname = hostname
         self.__user_code = user_code
-        self.__user_id = user_id
+        self.__app_id = app_id
         self.__panel_id = panel_id
         self.__partition = partition
         self.__user_email = user_email
@@ -502,12 +555,13 @@ class API(object):
         self.__url_alerts = self.__url_base + '/alerts'
         self.__url_troubles = self.__url_base + '/troubles'
 
-        self.__url_general_panel_info = self.__url_base + '/panel_info'
+        self.__url_panel_info = self.__url_base + '/panel_info'
         self.__url_events = self.__url_base + '/events'
         self.__url_wakeup_sms = self.__url_base + '/wakeup_sms'
         self.__url_all_devices = self.__url_base + '/devices'
         self.__url_set_state = self.__url_base + '/set_state'
         self.__url_locations = self.__url_base + '/locations'
+        self.__url_process_status = self.__url_base + '/process_status'
 
         # Create a new session
         self.__session = requests.session()
@@ -534,14 +588,29 @@ class API(object):
         if with_user_token:
             headers['User-Token'] = self.__user_token
 
-        # Perform the request and raise an exception
+        logging.debug('=== GET REQUEST -> ' + url + " ===")
+        logging.debug(headers)
+        logging.debug('=== END REQUEST ===')
+
+        # Perform the request and log an exception
         # if the response is not OK (HTML 200)
-        response = self.__session.get(url, headers=headers)
-        response.raise_for_status()
+        logging.debug('=== BEGIN RESPONSE ===')
+        try:
+            response = self.__session.get(url, headers=headers)
+            response.raise_for_status()
+        except requests.exceptions.HTTPError as err:
+            logging.error(err)
+            logging.error(response.content.decode('utf-8'))
 
         if response.status_code == requests.codes.ok:
-            value = json.loads(response.content.decode('utf-8'))
-            return value
+            resp = json.loads(response.content.decode('utf-8'))
+            logging.debug(resp)
+
+            logging.debug('=== END RESPONSE ===')
+            return resp
+        else:
+            logging.error(response.content.decode('utf-8'))
+            logging.debug('=== END RESPONSE ===')
 
     def __send_post_request(self, url, data_json, with_user_token, with_session_token):
         """ Send a POST request to the server. Includes the Session-Token
@@ -567,23 +636,30 @@ class API(object):
         if with_user_token:
             headers['User-Token'] = self.__user_token
 
-        logging.debug('POST REQUEST -> ' + url)
+        logging.debug('=== POST REQUEST -> ' + url + " ===")
         logging.debug(headers)
         logging.debug(data_json)
-
-        # Perform the request and raise an exception
+        logging.debug('=== END REQUEST ===')
+        
+        # Perform the request and log an exception
         # if the response is not OK (HTML 200)
-        response = self.__session.post(url, headers=headers, data=data_json)
-        response.raise_for_status()
+        logging.debug('=== BEGIN RESPONSE ===')
+        try:
+            response = self.__session.post(url, headers=headers, data=data_json)
+            response.raise_for_status()
+        except requests.exceptions.HTTPError as err:
+            logging.error(err)
+            logging.error(response.content.decode('utf-8'))
 
         # Check HTTP response code
         if response.status_code == requests.codes.ok:
             resp = json.loads(response.content.decode('utf-8'))
             logging.debug(resp)
-
+            logging.debug('=== END RESPONSE ===')
             return resp
         else:
-            logging.debug(response)
+            logging.error(response.content.decode('utf-8'))
+            logging.debug('=== END RESPONSE ===')
             return None
 
     ######################
@@ -611,9 +687,9 @@ class API(object):
         return self.__user_code
 
     @property
-    def user_id(self):
+    def app_id(self):
         """ Property to keep track of the user id (UUID) beeing used. """
-        return self.__user_id
+        return self.__app_id
 
     @property
     def panel_id(self):
@@ -634,11 +710,10 @@ class API(object):
     def login(self):
         """ Try to login and get a user token. """
 
-        # Setup authentication information (Used only in 8.0)
         login_info = {
             'email': self.__user_email,
             'password': self.__user_password,
-            'app_id': self.__user_id
+            'app_id': self.__app_id
         }
 
         login_json = json.dumps(login_info, separators=(',', ':'))
@@ -655,7 +730,7 @@ class API(object):
         panel_login_info = {
             'user_code': self.__user_code,
             'app_type': self.__app_type,
-            'app_id': self.__user_id,
+            'app_id': self.__app_id,
             'panel_serial': self.__panel_id
         }
         
@@ -682,16 +757,6 @@ class API(object):
 
     def get_alarms(self):
         """ Get the current alarms. """
-        
-        # Return List of: 
-        # alarm_type [string:ALARM_IN_MEMORY,TAMPER_MEMORY,HEAT_MEMORY,SMOKE_MEMORY,FIRE,PANIC,EMERGENCY,UNKNOWN]
-        # datetime [datetime]
-        # device_type [string]
-        # has_video [bool]
-        # location [string]
-        # partitions [List<partition>]
-        # zone [int]
-        # zone_type [string]
 
         return self.__send_get_request(self.__url_alarms,
                                        with_user_token=True,
@@ -700,14 +765,6 @@ class API(object):
     def get_troubles(self):
         """ Get the current troubles. """
 
-        # Return list of:
-        # device_type [string]
-        # location [string]
-        # partitions [List<partition>]
-        # trouble_type [string: TOO MANY - Important: INACTIVE,JAMMED,OPENED,TROUBLE,TAMPER,WENT_OFFLINE,LOW_BATTERY ]
-        # zone [int]
-        # zone_type [string]
-
         return self.__send_get_request(self.__url_troubles,
                                        with_user_token=True,
                                        with_session_token=True)
@@ -715,21 +772,13 @@ class API(object):
     def get_alerts(self):
         """ Get the current alerts. """
 
-        # Return list of:
-        # alert_type [string: BBA_WENT_OFFLINE,COLD_ALERT,FREEZER_ALERT,FREEZING_ALERT,GPRS_WENT_OFFLINE,HOT_ALERT,PLINK_WENT_OFFLINE,WENT_OFFLINE,UNKNOWN]
-        # device_type [string]
-        # location [string]
-        # partitions [List<partition>]
-        # zone [int]
-        # zone_type [string]
-
         return self.__send_get_request(self.__url_alerts,
                                        with_user_token=True,
                                        with_session_token=True)
 
-    def get_general_panel_info(self):
-        """ Get the general panel information. """
-        return self.__send_get_request(self.__url_general_panel_info,
+    def get_panel_info(self):
+        """ Get the panel information. """
+        return self.__send_get_request(self.__url_panel_info,
                                        with_user_token=True,
                                        with_session_token=True)
 
@@ -748,6 +797,7 @@ class API(object):
 
     def get_all_devices(self):
         """ Get the device specific information. """
+
         return self.__send_get_request(self.__url_all_devices,
                                        with_user_token=True,
                                        with_session_token=True)
@@ -767,8 +817,17 @@ class API(object):
         arm_json = json.dumps(arm_info, separators=(',', ':'))
 
         return self.__send_post_request(self.__url_set_state, arm_json,
-                                        with_user_token=True,
-                                        with_session_token=True)
+                                       with_user_token=True,
+                                       with_session_token=True)
+
+    def get_process_status(self, token):
+        logging.debug(token)
+
+        res = self.__send_get_request(self.__url_process_status + '?process_tokens=' + token,
+                                       with_user_token=True,
+                                       with_session_token=True)
+        logging.debug(res)
+        return res[0]['status']
 
     def arm_away(self, partition):
         """ Arm in Away mode and with Exit Delay. """
